@@ -42,16 +42,16 @@
 #include <stdlib.h>
 #include <string.h>
 #include <ctype.h>
-#include <inttypes.h>
 
 #include "nasm.h"
 #include "nasmlib.h"
+#include "error.h"
 #include "saa.h"
 #include "raa.h"
 #include "stdscan.h"
 #include "eval.h"
-#include "output/outform.h"
-#include "output/outlib.h"
+#include "outform.h"
+#include "outlib.h"
 
 #if defined OF_AOUT || defined OF_AOUTB
 
@@ -188,7 +188,7 @@ static void aout_init(void)
 
 #ifdef OF_AOUTB
 
-extern struct ofmt of_aoutb;
+extern const struct ofmt of_aoutb;
 
 static void aoutb_init(void)
 {
@@ -198,15 +198,15 @@ static void aoutb_init(void)
     is_pic = 0x00;              /* may become 0x40 */
 
     aout_gotpc_sect = seg_alloc();
-    define_label("..gotpc", aout_gotpc_sect + 1, 0L, NULL, false, false);
+    backend_label("..gotpc", aout_gotpc_sect + 1, 0L);
     aout_gotoff_sect = seg_alloc();
-    define_label("..gotoff", aout_gotoff_sect + 1, 0L, NULL, false, false);
+    backend_label("..gotoff", aout_gotoff_sect + 1, 0L);
     aout_got_sect = seg_alloc();
-    define_label("..got", aout_got_sect + 1, 0L, NULL, false, false);
+    backend_label("..got", aout_got_sect + 1, 0L);
     aout_plt_sect = seg_alloc();
-    define_label("..plt", aout_plt_sect + 1, 0L, NULL, false, false);
+    backend_label("..plt", aout_plt_sect + 1, 0L);
     aout_sym_sect = seg_alloc();
-    define_label("..sym", aout_sym_sect + 1, 0L, NULL, false, false);
+    backend_label("..sym", aout_sym_sect + 1, 0L);
 }
 
 #endif
@@ -244,11 +244,10 @@ static int32_t aout_section_names(char *name, int pass, int *bits)
     /*
      * Default to 32 bits.
      */
-    if (!name)
+    if (!name) {
         *bits = 32;
-
-    if (!name)
         return stext.index;
+    }
 
     if (!strcmp(name, ".text"))
         return stext.index;
@@ -895,59 +894,60 @@ static int32_t aout_segbase(int32_t segment)
     return segment;
 }
 
-static void aout_filename(char *inname, char *outname)
-{
-    standard_extension(inname, outname, ".o");
-}
-
 extern macros_t aout_stdmac[];
 
 #endif                          /* OF_AOUT || OF_AOUTB */
 
 #ifdef OF_AOUT
 
-struct ofmt of_aout = {
+const struct ofmt of_aout = {
     "Linux a.out object files",
     "aout",
+    ".o",
     0,
     32,
     null_debug_arr,
     &null_debug_form,
     aout_stdmac,
     aout_init,
-    null_setinfo,
+    null_reset,
+    nasm_do_legacy_output,
     aout_out,
     aout_deflabel,
     aout_section_names,
+    NULL,
     null_sectalign,
     aout_segbase,
     null_directive,
-    aout_filename,
-    aout_cleanup
+    aout_cleanup,
+    NULL                        /* pragma list */
 };
 
 #endif
 
 #ifdef OF_AOUTB
 
-struct ofmt of_aoutb = {
+const struct ofmt of_aoutb = {
     "NetBSD/FreeBSD a.out object files",
     "aoutb",
+    ".o",
     0,
     32,
     null_debug_arr,
     &null_debug_form,
     aout_stdmac,
     aoutb_init,
-    null_setinfo,
+    null_reset,
+    nasm_do_legacy_output,
     aout_out,
     aout_deflabel,
     aout_section_names,
+    NULL,
     null_sectalign,
     aout_segbase,
     null_directive,
-    aout_filename,
-    aout_cleanup
+    aout_cleanup,
+    NULL                        /* pragma list */
 };
 
 #endif
