@@ -1,6 +1,6 @@
 /* ----------------------------------------------------------------------- *
  *
- *   Copyright 1996-2014 The NASM Authors - All Rights Reserved
+ *   Copyright 1996-2017 The NASM Authors - All Rights Reserved
  *   See the file AUTHORS included with the NASM distribution for
  *   the specific copyright holders.
  *
@@ -43,16 +43,17 @@
 #include <string.h>
 #include <ctype.h>
 #include <time.h>
-#include <inttypes.h>
 
 #include "nasm.h"
 #include "nasmlib.h"
+#include "ilog2.h"
+#include "error.h"
 #include "saa.h"
 #include "raa.h"
 #include "eval.h"
-#include "output/outform.h"
-#include "output/outlib.h"
-#include "output/pecoff.h"
+#include "outform.h"
+#include "outlib.h"
+#include "pecoff.h"
 
 #if defined(OF_COFF) || defined(OF_WIN32) || defined(OF_WIN64)
 
@@ -105,9 +106,6 @@ bool win32, win64;
 static int32_t imagebase_sect;
 #define WRT_IMAGEBASE "..imagebase"
 
-char coff_infile[FILENAME_MAX];
-char coff_outfile[FILENAME_MAX];
-
 /*
  * Some common section flags by default
  */
@@ -200,7 +198,7 @@ static void coff_win64_init(void)
     win64 = true;
     coff_gen_init();
     imagebase_sect = seg_alloc()+1;
-    define_label(WRT_IMAGEBASE, imagebase_sect, 0, NULL, false, false);
+    backend_label(WRT_IMAGEBASE, imagebase_sect, 0);
 }
 
 static void coff_std_init(void)
@@ -307,10 +305,9 @@ static int32_t coff_section_names(char *name, int pass, int *bits)
             *bits = 64;
         else
             *bits = 32;
-    }
 
-    if (!name)
         return def_seg;
+    }
 
     p = name;
     while (*p && !nasm_isspace(*p))
@@ -769,7 +766,8 @@ static void BuildExportTable(STRING **rvp)
     *rvp = NULL;
 }
 
-static int coff_directives(enum directives directive, char *value, int pass)
+static enum directive_result
+coff_directives(enum directive directive, char *value, int pass)
 {
     switch (directive) {
     case D_EXPORT:
@@ -777,7 +775,7 @@ static int coff_directives(enum directives directive, char *value, int pass)
         char *q, *name;
 
         if (pass == 2)
-            return 1;           /* ignore in pass two */
+            return DIRR_OK;           /* ignore in pass two */
         name = q = value;
         while (*q && !nasm_isspace(*q))
             q++;
@@ -789,14 +787,14 @@ static int coff_directives(enum directives directive, char *value, int pass)
 
         if (!*name) {
             nasm_error(ERR_NONFATAL, "`export' directive requires export name");
-            return 1;
+            return DIRR_ERROR;
         }
         if (*q) {
             nasm_error(ERR_NONFATAL, "unrecognized export qualifier `%s'", q);
-            return 1;
+            return DIRR_ERROR;
         }
         AddExport(name);
-        return 1;
+        return DIRR_OK;
     }
     case D_SAFESEH:
     {
@@ -856,12 +854,13 @@ static int coff_directives(enum directives directive, char *value, int pass)
             if (n == coff_nsyms) {
                 nasm_error(ERR_NONFATAL,
                            "`safeseh' directive requires valid symbol");
+                return DIRR_ERROR;
             }
         }
-        return 1;
+        return DIRR_OK;
     }
     default:
-        return 0;
+        return DIRR_UNKNOWN;
     }
 }
 
@@ -1078,7 +1077,7 @@ static void coff_write_symbols(void)
      * The `.file' record, and the file name auxiliary record.
      */
     coff_symbol(".file", 0L, 0L, -2, 0, 0x67, 1);
-    strncpy(filename, coff_infile, 18);
+    strncpy(filename, inname, 18);
     nasm_write(filename, 18, ofile);
 
     /*
@@ -1141,104 +1140,93 @@ static int32_t coff_segbase(int32_t segment)
     return segment;
 }
 
-static void coff_std_filename(char *inname, char *outname)
-{
-    strcpy(coff_infile, inname);
-    standard_extension(inname, outname, ".o");
-    strcpy(coff_outfile, outname);
-}
-
-static void coff_win32_filename(char *inname, char *outname)
-{
-    strcpy(coff_infile, inname);
-    standard_extension(inname, outname, ".obj");
-    strcpy(coff_outfile, outname);
-}
-
 extern macros_t coff_stdmac[];
 
-static int coff_set_info(enum geninfo type, char **val)
-{
-    (void)type;
-    (void)val;
-    return 0;
-}
 #endif /* defined(OF_COFF) || defined(OF_WIN32) */
 
 #ifdef OF_COFF
 
-struct ofmt of_coff = {
+const struct ofmt of_coff = {
     "COFF (i386) object files (e.g. DJGPP for DOS)",
     "coff",
+    ".o",
     0,
     32,
     null_debug_arr,
     &null_debug_form,
     coff_stdmac,
     coff_std_init,
-    coff_set_info,
+    null_reset,
+    nasm_do_legacy_output,
     coff_out,
     coff_deflabel,
     coff_section_names,
+    NULL,
     coff_sectalign,
     coff_segbase,
     coff_directives,
-    coff_std_filename,
-    coff_cleanup
+    coff_cleanup,
+    NULL                        /* pragma list */
 };
 
 #endif
 
-extern struct dfmt df_cv8;
+extern const struct dfmt df_cv8;
 
 #ifdef OF_WIN32
 
-struct dfmt *win32_debug_arr[2] = { &df_cv8, NULL };
+static const struct dfmt * const win32_debug_arr[2] = { &df_cv8, NULL };
 
-struct ofmt of_win32 = {
+const struct ofmt of_win32 = {
     "Microsoft Win32 (i386) object files",
     "win32",
+    ".obj",
     0,
     32,
     win32_debug_arr,
     &df_cv8,
     coff_stdmac,
     coff_win32_init,
-    coff_set_info,
+    null_reset,
+    nasm_do_legacy_output,
     coff_out,
     coff_deflabel,
     coff_section_names,
+    NULL,
     coff_sectalign,
     coff_segbase,
     coff_directives,
-    coff_win32_filename,
-    coff_cleanup
+    coff_cleanup,
+    NULL                        /* pragma list */
 };
 
 #endif
 
 #ifdef OF_WIN64
 
-struct dfmt *win64_debug_arr[2] = { &df_cv8, NULL };
+static const struct dfmt * const win64_debug_arr[2] = { &df_cv8, NULL };
 
-struct ofmt of_win64 = {
+const struct ofmt of_win64 = {
     "Microsoft Win64 (x86-64) object files",
     "win64",
+    ".obj",
     0,
     64,
     win64_debug_arr,
     &df_cv8,
     coff_stdmac,
     coff_win64_init,
-    coff_set_info,
+    null_reset,
+    nasm_do_legacy_output,
     coff_out,
     coff_deflabel,
     coff_section_names,
+    NULL,
     coff_sectalign,
     coff_segbase,
     coff_directives,
-    coff_win32_filename,
-    coff_cleanup
+    coff_cleanup,
+    NULL                        /* pragma list */
 };
 
 #endif
