@@ -1,6 +1,6 @@
 /* ----------------------------------------------------------------------- *
  *   
- *   Copyright 1996-2013 The NASM Authors - All Rights Reserved
+ *   Copyright 1996-2016 The NASM Authors - All Rights Reserved
  *   See the file AUTHORS included with the NASM distribution for
  *   the specific copyright holders.
  *
@@ -44,20 +44,20 @@
 #include <string.h>
 #include <ctype.h>
 #include <assert.h>
-#include <inttypes.h>
 
 #include "nasm.h"
 #include "nasmlib.h"
+#include "error.h"
 #include "saa.h"
-#include "output/outform.h"
-#include "output/outlib.h"
+#include "outform.h"
+#include "outlib.h"
 
 /* VERBOSE_WARNINGS: define this to add some extra warnings... */
 #define VERBOSE_WARNINGS
 
 #ifdef OF_RDF2
 
-#include "rdoff/rdoff.h"
+#include "rdoff.h"
 
 /* This signature is written to start of RDOFF files */
 static const char *RDOFF2Id = RDOFF2_SIGNATURE;
@@ -720,7 +720,8 @@ static int32_t rdf2_segbase(int32_t segment)
 /*
  * Handle RDOFF2 specific directives
  */
-static int rdf2_directive(enum directives directive, char *value, int pass)
+static enum directive_result
+rdf2_directive(enum directive directive, char *value, int pass)
 {
     size_t n;
 
@@ -729,7 +730,7 @@ static int rdf2_directive(enum directives directive, char *value, int pass)
 	n = strlen(value);
 	if (n >= MODLIB_NAME_MAX) {
 	    nasm_error(ERR_NONFATAL, "name size exceeds %d bytes", MODLIB_NAME_MAX);
-	    return 1;
+	    return DIRR_ERROR;
 	}
         if (pass == 1) {
             struct DLLRec r;
@@ -738,12 +739,12 @@ static int rdf2_directive(enum directives directive, char *value, int pass)
             strcpy(r.libname, value);
             write_dll_rec(&r);
         }
-        return 1;
+        return DIRR_OK;
 	
     case D_MODULE:
 	if ((n = strlen(value)) >= MODLIB_NAME_MAX) {
 	    nasm_error(ERR_NONFATAL, "name size exceeds %d bytes", MODLIB_NAME_MAX);
-	    return 1;
+	    return DIRR_ERROR;
 	}
         if (pass == 1) {
             struct ModRec r;
@@ -752,45 +753,36 @@ static int rdf2_directive(enum directives directive, char *value, int pass)
             strcpy(r.modname, value);
             write_modname_rec(&r);
         }
-        return 1;
+        return DIRR_OK;
 
     default:
-	return 0;
+	return DIRR_UNKNOWN;
     }
 }
 
-static void rdf2_filename(char *inname, char *outname)
-{
-    standard_extension(inname, outname, ".rdf");
-}
-
 extern macros_t rdf2_stdmac[];
 
-static int rdf2_set_info(enum geninfo type, char **val)
-{
-    (void)type;
-    (void)val;
-    return 0;
-}
-
-struct ofmt of_rdf2 = {
+const struct ofmt of_rdf2 = {
     "Relocatable Dynamic Object File Format v2.0",
     "rdf",
+    ".rdf",
     0,
     64,
     null_debug_arr,
     &null_debug_form,
     rdf2_stdmac,
     rdf2_init,
-    rdf2_set_info,
+    null_reset,
+    nasm_do_legacy_output,
     rdf2_out,
     rdf2_deflabel,
     rdf2_section_names,
+    NULL,
     null_sectalign,
     rdf2_segbase,
     rdf2_directive,
-    rdf2_filename,
-    rdf2_cleanup
+    rdf2_cleanup,
+    NULL                        /* pragma list */
 };
 
 #endif                          /* OF_RDF2 */
